﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Data;

namespace EditableObjects
{
    public class DataObject : DataObjectBase
    {
        public enum EnumValue
        {
            Red,
            Green,
            Blue,
            Orange,
            Purple,
            Yellow
        }

        private EnumValue _icon;
        private string _name;
        private int _amount;
        DateTime _from, _till;

        public DataObject(string name, EnumValue icon, int amount, DateTime from, DateTime till)
        {
            _name = name;
            Icon = icon;
            Amount = amount;
            From = from;
            Till = till;
        }

        public EnumValue Icon
        {
            get { return _icon; }
            set
            {
                this.Trace(string.Format("DataObject.Icon set property to: {0}.", value));

                _icon = value;

                ValidateIcon();

                OnPropertyChanged("Icon");
            }
        }

        public int Amount
        {
            get { return _amount; }
            set
            {
                this.Trace(string.Format("DataObject.Amount set property to: {0}.", value));

                _amount = value;

                ValidateAmount();

                OnPropertyChanged("Amount");
            }
        }

        public string Name
        {
            get { return _name; }
        }

        public DateTime From
        {
            get { return _from; }
            set
            {
                this.Trace(string.Format("DataObject.From set property to: {0}.", value));

                _from = value;

                ValidateFromAndTill();

                OnPropertyChanged("From");
            }
        }

        public DateTime Till
        {
            get { return _till; }
            set
            {
                this.Trace(string.Format("DataObject.Till set property to: {0}.", value));

                _till = value;

                ValidateFromAndTill();

                OnPropertyChanged("Till");
            }
        }

        private void ValidateIcon()
        {
            base.ClearErrors("Icon");

            if (_icon == EnumValue.Red)
                base.AddError("Icon", "Red is not allowed!");
        }

        private void ValidateAmount()
        {
            base.ClearErrors("Amount");

            if (_amount <= 0)
                base.AddError("Amount", "Amount must be greater than 0!");
        }

        private void ValidateFromAndTill()
        {
            base.ClearErrors("From");
            base.ClearErrors("Till");

            if (_from.CompareTo(_till) > 0)
            {
                base.AddError("From", "From must be before till!");
                base.AddError("Till", "From must be before till!");
            }
        }

        protected override void ExportValues(Dictionary<string, object> backup)
        {
            backup.Add("Icon", Icon);
            backup.Add("Amount", Amount);
            backup.Add("From", From);
            backup.Add("Till", Till);
        }

        protected override void ImportValues(Dictionary<string, object> backup)
        {
            Icon = (EnumValue)backup["Icon"];
            Amount = (int)backup["Amount"];
            From = (DateTime)backup["From"];
            Till = (DateTime)backup["Till"];
        }
    }
}
